"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
// import { Parser } from "xml2js";
// import * as xmlfmt from "xml-formatter";
const entity_parse_1 = require("./xdml/entity-parse");
const formatter_1 = require("./xdml/formatter");
let connection = vscode_languageserver_1.createConnection(vscode_languageserver_1.ProposedFeatures.all);
let documents = new vscode_languageserver_1.TextDocuments();
let hasConfigurationCapability = false;
let hasWorkspaceFolderCapability = false;
let hasDiagnosticRelatedInformationCapability = false;
connection.onInitialize((params) => {
    let capabilities = params.capabilities;
    // Does the client support the `workspace/configuration` request?
    // If not, we will fall back using global settings
    hasConfigurationCapability = !!(capabilities.workspace && !!capabilities.workspace.configuration);
    hasWorkspaceFolderCapability = !!(capabilities.workspace && !!capabilities.workspace.workspaceFolders);
    hasDiagnosticRelatedInformationCapability = !!(capabilities.textDocument &&
        capabilities.textDocument.publishDiagnostics &&
        capabilities.textDocument.publishDiagnostics.relatedInformation);
    return {
        capabilities: {
            hoverProvider: true,
            documentFormattingProvider: true,
            textDocumentSync: documents.syncKind,
            // Tell the client that the server supports code completion
            completionProvider: {
                resolveProvider: true
            }
        }
    };
});
connection.onInitialized(() => {
    if (hasConfigurationCapability) {
        connection.client.register(vscode_languageserver_1.DidChangeConfigurationNotification.type, undefined);
    }
    if (hasWorkspaceFolderCapability) {
        connection.workspace.onDidChangeWorkspaceFolders(_event => {
            connection.console.log("Workspace folder change event received.");
        });
    }
});
// The global settings, used when the `workspace/configuration` request is not supported by the client.
// Please note that this is not the case when using this server with the client provided in this example
// but could happen with other clients.
const defaultSettings = { validation: true };
let globalSettings = defaultSettings;
// Cache the settings of all open documents
let documentSettings = new Map();
// let xmlParser = new Parser({ strict: false });
connection.onDidChangeConfiguration(change => {
    if (hasConfigurationCapability) {
        // Reset all cached document settings
        documentSettings.clear();
    }
    else {
        globalSettings = (change.settings.xdml || defaultSettings);
    }
    // Revalidate all open text documents
    documents.all().forEach(validateTextDocument);
});
function getDocumentSettings(resource) {
    if (!hasConfigurationCapability) {
        return Promise.resolve(globalSettings);
    }
    let result = documentSettings.get(resource);
    if (!result) {
        result = connection.workspace.getConfiguration({
            scopeUri: resource,
            section: "xdml"
        });
        documentSettings.set(resource, result);
    }
    return result;
}
// Only keep settings for open documents
documents.onDidClose(e => {
    documentSettings.delete(e.document.uri);
});
connection.onHover((e) => __awaiter(void 0, void 0, void 0, function* () {
    var doc = documents.get(e.textDocument.uri);
    var position = e.position;
    const result = entity_parse_1.getCurrentEntityInfos(doc, position);
    if (result === null) {
        return {
            contents: []
        };
    }
    const { type, nsUri, name, metainfo } = result;
    let tokenType = "Token";
    switch (type) {
        case entity_parse_1.ElementType.DartClass:
            tokenType = "Dart类";
            break;
        case entity_parse_1.ElementType.DartFactory:
            tokenType = "Dart类工厂";
            break;
        case entity_parse_1.ElementType.DartProperty:
            tokenType = "Dart类属性";
            break;
        case entity_parse_1.ElementType.XDMLVirtualVariable:
            tokenType = "XDML虚拟变量声明";
            break;
        case entity_parse_1.ElementType.XDMLNode:
            tokenType = "XDML语法结构";
            break;
        case entity_parse_1.ElementType.XDMLAttr:
            tokenType = "XDML结构属性";
            break;
        case entity_parse_1.ElementType.NamespaceAttrDefine:
            tokenType = "XDML名称空间声明";
            break;
        default:
            tokenType = "Token";
    }
    console.log(metainfo);
    return {
        contents: [`**${tokenType}** - ${"`"}${name}${"`"}`]
            .concat(metainfo.pointList.map(i => ` - **${i.k}** - ${i.v}`))
            .join("\n\n")
    };
}));
// The content of a text document has changed. This event is emitted
// when the text document first opened or when its content has changed.
documents.onDidChangeContent(change => {
    validateTextDocument(change.document);
});
documents.onWillSave((e) => __awaiter(void 0, void 0, void 0, function* () {
    const token = new vscode_languageserver_1.RequestType("textDocument/formatting");
    connection.sendRequest(token, {
        textDocument: vscode_languageserver_1.TextDocumentIdentifier.create(e.document.uri),
        options: {
            tabSize: 2,
            insertSpaces: true
        }
    });
}));
connection.onDocumentFormatting((e) => __awaiter(void 0, void 0, void 0, function* () {
    const doc = documents.get(e.textDocument.uri);
    const text = doc.getText();
    const lastPosition = doc.positionAt(text.length);
    const result = yield formatter_1.formatXml(text);
    return [
        vscode_languageserver_1.TextEdit.replace(vscode_languageserver_1.Range.create(vscode_languageserver_1.Position.create(0, 0), lastPosition), result)
    ];
}));
function validateTextDocument(textDocument) {
    return __awaiter(this, void 0, void 0, function* () {
        // In this simple example we get the settings for every validate run.
        let settings = yield getDocumentSettings(textDocument.uri);
        // The validator creates diagnostics for all uppercase words length 2 and more
        let text = textDocument.getText();
        // let pattern = /\b[A-Z]{2,}\b/g;
        // let m: RegExpExecArray | null;
        // let problems = 0;
        // let diagnostics: Diagnostic[] = [];
        // while (settings.validation && (m = pattern.exec(text)) && problems < 100) {
        //   problems++;
        //   let diagnostic: Diagnostic = {
        //     severity: DiagnosticSeverity.Warning,
        //     range: {
        //       start: textDocument.positionAt(m.index),
        //       end: textDocument.positionAt(m.index + m[0].length)
        //     },
        //     message: `${m[0]} is all uppercase.`,
        //     source: "ex"
        //   };
        //   if (hasDiagnosticRelatedInformationCapability) {
        //     diagnostic.relatedInformation = [
        //       {
        //         location: {
        //           uri: textDocument.uri,
        //           range: Object.assign({}, diagnostic.range)
        //         },
        //         message: "Spelling matters"
        //       },
        //       {
        //         location: {
        //           uri: textDocument.uri,
        //           range: Object.assign({}, diagnostic.range)
        //         },
        //         message: "Particularly for names"
        //       }
        //     ];
        //   }
        //   diagnostics.push(diagnostic);
        // }
        // // Send the computed diagnostics to VSCode.
        // connection.sendDiagnostics({ uri: textDocument.uri, diagnostics });
    });
}
connection.onDidChangeWatchedFiles(_change => {
    // Monitored files have change in VSCode
    connection.console.log("We received an file change event");
});
// This handler provides the initial list of the completion items.
connection.onCompletion((_textDocumentPosition) => {
    // The pass parameter contains the position of the text document in
    // which code complete got requested. For the example we ignore this
    // info and always provide the same completion items.
    return [
        {
            label: "TypeScript",
            kind: vscode_languageserver_1.CompletionItemKind.Text,
            data: 1
        },
        {
            label: "JavaScript",
            kind: vscode_languageserver_1.CompletionItemKind.Text,
            data: 2
        }
    ];
});
// This handler resolves additional information for the item selected in
// the completion list.
connection.onCompletionResolve((item) => {
    if (item.data === 1) {
        item.detail = "TypeScript details";
        item.documentation = "TypeScript documentation";
    }
    else if (item.data === 2) {
        item.detail = "JavaScript details";
        item.documentation = "JavaScript documentation";
    }
    return item;
});
/*
connection.onDidOpenTextDocument((params) => {
    // A text document got opened in VSCode.
    // params.uri uniquely identifies the document. For documents store on disk this is a file URI.
    // params.text the initial full content of the document.
    connection.console.log(`${params.textDocument.uri} opened.`);
});
connection.onDidChangeTextDocument((params) => {
    // The content of a text document did change in VSCode.
    // params.uri uniquely identifies the document.
    // params.contentChanges describe the content changes to the document.
    connection.console.log(`${params.textDocument.uri} changed: ${JSON.stringify(params.contentChanges)}`);
});
connection.onDidCloseTextDocument((params) => {
    // A text document got closed in VSCode.
    // params.uri uniquely identifies the document.
    connection.console.log(`${params.textDocument.uri} closed.`);
});
*/
// Make the text document manager listen on the connection
// for open, change and close text document events
documents.listen(connection);
// Listen on the connection
connection.listen();
//# sourceMappingURL=server.js.map